@@ -16,6 +16,8 @@
  * Foundation, Inc., 51 Franklin Street - Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
+#include <string.h>
+
 #include <goo/GooList.h>
 #include <splash/SplashBitmap.h>
 #include <GlobalParams.h>
@@ -28,6 +30,7 @@
 #include <Stream.h>
 #include <FontInfo.h>
 #include <PDFDocEncoding.h>
+#include <OptionalContent.h>
 
 #include "poppler.h"
 #include "poppler-private.h"
@@ -37,6 +40,8 @@ enum {
 	PROP_0,
 	PROP_TITLE,
 	PROP_FORMAT,
+	PROP_FORMAT_MAJOR,
+	PROP_FORMAT_MINOR,
 	PROP_AUTHOR,
 	PROP_SUBJECT,
 	PROP_KEYWORDS,
@@ -52,37 +57,60 @@ enum {
 	PROP_METADATA
 };
 
+static void poppler_document_layers_free (PopplerDocument *document);
+
 typedef struct _PopplerDocumentClass PopplerDocumentClass;
 struct _PopplerDocumentClass
 {
   GObjectClass parent_class;
 };
 
-G_DEFINE_TYPE (PopplerDocument, poppler_document, G_TYPE_OBJECT);
+G_DEFINE_TYPE (PopplerDocument, poppler_document, G_TYPE_OBJECT)
 
 static PopplerDocument *
 _poppler_document_new_from_pdfdoc (PDFDoc  *newDoc,
                                    GError **error)
 {
   PopplerDocument *document;
-  int err;
 
   document = (PopplerDocument *) g_object_new (POPPLER_TYPE_DOCUMENT, NULL, NULL);
 
   if (!newDoc->isOk()) {
-    err = newDoc->getErrorCode();
-    delete newDoc;
-    if (err == errEncrypted) {
-      g_set_error (error, POPPLER_ERROR,
-		   POPPLER_ERROR_ENCRYPTED,
-		   "Document is encrypted.");
-    } else {
-      g_set_error (error, G_FILE_ERROR,
-		   G_FILE_ERROR_FAILED,
-		   "Failed to load document from data (error %d)'\n",
-		   err);
+    int fopen_errno;
+    switch (newDoc->getErrorCode())
+      {
+      case errOpenFile:
+        // If there was an error opening the file, count it as a G_FILE_ERROR 
+        // and set the GError parameters accordingly. (this assumes that the 
+        // only way to get an errOpenFile error is if newDoc was created using 
+        // a filename and thus fopen was called, which right now is true.
+        fopen_errno = newDoc->getFopenErrno();
+        g_set_error (error, G_FILE_ERROR,
+		     g_file_error_from_errno (fopen_errno),
+		     "%s", g_strerror (fopen_errno));
+	break;
+      case errBadCatalog:
+        g_set_error (error, POPPLER_ERROR,
+		     POPPLER_ERROR_BAD_CATALOG,
+		     "Failed to read the document catalog");
+	break;
+      case errDamaged:
+        g_set_error (error, POPPLER_ERROR,
+		     POPPLER_ERROR_DAMAGED,
+		     "PDF document is damaged");
+	break;
+      case errEncrypted:
+        g_set_error (error, POPPLER_ERROR,
+		     POPPLER_ERROR_ENCRYPTED,
+		     "Document is encrypted");
+	break;
+      default:
+        g_set_error (error, POPPLER_ERROR,
+		     POPPLER_ERROR_INVALID,
+		     "Failed to load document");
     }
-
+    
+    delete newDoc;
     return NULL;
   }
 
@@ -90,14 +118,15 @@ _poppler_document_new_from_pdfdoc (PDFDoc  *newDoc,
 
 #if defined (HAVE_CAIRO)
   document->output_dev = new CairoOutputDev ();
+  document->output_dev->startDoc(document->doc->getXRef (), document->doc->getCatalog ());
 #elif defined (HAVE_SPLASH)
   SplashColor white;
   white[0] = 255;
   white[1] = 255;
   white[2] = 255;
   document->output_dev = new SplashOutputDev(splashModeRGB8, 4, gFalse, white);
-#endif
   document->output_dev->startDoc(document->doc->getXRef ());
+#endif
 
   return document;
 }
@@ -122,7 +151,6 @@ poppler_document_new_from_file (const char  *uri,
   PDFDoc *newDoc;
   GooString *filename_g;
   GooString *password_g;
-  int err;
   char *filename;
 
   if (!globalParams) {
@@ -133,16 +161,43 @@ poppler_document_new_from_file (const char  *uri,
   if (!filename)
     return NULL;
 
+  password_g = NULL;
+  if (password != NULL) {
+    if (g_utf8_validate (password, -1, NULL)) {
+      gchar *password_latin;
+      
+      password_latin = g_convert (password, -1,
+				  "ISO-8859-1",
+				  "UTF-8",
+				  NULL, NULL, NULL);
+      password_g = new GooString (password_latin);
+      g_free (password_latin);
+    } else {
+      password_g = new GooString (password);
+    }
+  }
+
+#ifdef G_OS_WIN32
+  wchar_t *filenameW;
+  int length;
+
+  length = MultiByteToWideChar(CP_UTF8, 0, filename, -1, NULL, 0);
+
+  filenameW = new WCHAR[length];
+  if (!filenameW)
+      return NULL;
+
+  length = MultiByteToWideChar(CP_UTF8, 0, filename, -1, filenameW, length);
+
+  newDoc = new PDFDoc(filenameW, length, password_g, password_g);
+  delete filenameW;
+#else
   filename_g = new GooString (filename);
+  newDoc = new PDFDoc(filename_g, password_g, password_g);
+#endif
   g_free (filename);
 
-  password_g = NULL;
-  if (password != NULL)
-    password_g = new GooString (password);
-
-  newDoc = new PDFDoc(filename_g, password_g, password_g);
-  if (password_g)
-    delete password_g;
+  delete password_g;
 
   return _poppler_document_new_from_pdfdoc (newDoc, error);
 }
@@ -170,8 +225,6 @@ poppler_document_new_from_data (char        *data,
   PDFDoc *newDoc;
   MemStream *str;
   GooString *password_g;
-  int err;
-  char *filename;
 
   if (!globalParams) {
     globalParams = new GlobalParams();
@@ -186,19 +239,47 @@ poppler_document_new_from_data (char        *data,
     password_g = new GooString (password);
 
   newDoc = new PDFDoc(str, password_g, password_g);
-  if (password_g)
-    delete password_g;
+  delete password_g;
 
   return _poppler_document_new_from_pdfdoc (newDoc, error);
 }
 
+static gboolean
+handle_save_error (int      err_code,
+		   GError **error)
+{
+  switch (err_code)
+    {
+    case errNone:
+      break;
+    case errOpenFile:
+      g_set_error (error, POPPLER_ERROR,
+		   POPPLER_ERROR_OPEN_FILE,
+		   "Failed to open file for writing");
+      break;
+    case errEncrypted:
+      g_set_error (error, POPPLER_ERROR,
+		   POPPLER_ERROR_ENCRYPTED,
+		   "Document is encrypted");
+      break;
+    default:
+      g_set_error (error, POPPLER_ERROR,
+		   POPPLER_ERROR_INVALID,
+		   "Failed to save document");
+    }
+
+  return err_code == errNone;
+}
+
 /**
  * poppler_document_save:
  * @document: a #PopplerDocument
  * @uri: uri of file to save
  * @error: return location for an error, or %NULL
  * 
- * Saves @document.  If @error is set, %FALSE will be returned. Possible errors
+ * Saves @document. Any change made in the document such as 
+ * form fields filled by the user will be saved. 
+ * If @error is set, %FALSE will be returned. Possible errors
  * include those in the #G_FILE_ERROR domain.
  * 
  * Return value: %TRUE, if the document was successfully saved
@@ -216,10 +297,49 @@ poppler_document_save (PopplerDocument  *document,
   filename = g_filename_from_uri (uri, NULL, error);
   if (filename != NULL) {
     GooString *fname = new GooString (filename);
+    int err_code;
     g_free (filename);
 
-    retval = document->doc->saveAs (fname);
+    err_code = document->doc->saveAs (fname);
+    retval = handle_save_error (err_code, error);
+    delete fname;
+  }
 
+  return retval;
+}
+
+/**
+ * poppler_document_save_a_copy:
+ * @document: a #PopplerDocument
+ * @uri: uri of file to save
+ * @error: return location for an error, or %NULL
+ * 
+ * Saves a copy of the original @document.
+ * Any change made in the document such as 
+ * form fields filled by the user will not be saved. 
+ * If @error is set, %FALSE will be returned. Possible errors
+ * include those in the #G_FILE_ERROR domain.
+ * 
+ * Return value: %TRUE, if the document was successfully saved
+ **/
+gboolean
+poppler_document_save_a_copy (PopplerDocument  *document,
+			      const char       *uri,
+			      GError          **error)
+{
+  char *filename;
+  gboolean retval = FALSE;
+
+  g_return_val_if_fail (POPPLER_IS_DOCUMENT (document), FALSE);
+
+  filename = g_filename_from_uri (uri, NULL, error);
+  if (filename != NULL) {
+    GooString *fname = new GooString (filename);
+    int err_code;
+    g_free (filename);
+
+    err_code = document->doc->saveWithoutChangesAs (fname);
+    retval = handle_save_error (err_code, error);
     delete fname;
   }
 
@@ -231,6 +351,7 @@ poppler_document_finalize (GObject *object)
 {
   PopplerDocument *document = POPPLER_DOCUMENT (object);
 
+  poppler_document_layers_free (document);
   delete document->output_dev;
   delete document->doc;
 }
@@ -363,6 +484,10 @@ poppler_document_get_attachments (PopplerDocument *document)
       EmbFile *emb_file;
 
       emb_file = catalog->embeddedFile (i);
+      if (!emb_file->isOk ()) {
+        delete emb_file;
+	continue;
+      }
       attachment = _poppler_attachment_new (document, emb_file);
       delete emb_file;
 
@@ -463,7 +588,7 @@ static void
 info_dict_get_date (Dict *info_dict, const gchar *key, GValue *value) 
 {
   Object obj;
-  GTime result;
+  time_t result;
 
   if (!info_dict->lookup ((gchar *)key, &obj)->isString ()) {
     obj.free ();
@@ -543,9 +668,15 @@ poppler_document_get_property (GObject    *object,
     case PROP_FORMAT:
       str = g_strndup("PDF-", 15); /* allocates 16 chars, pads with \0s */
       g_ascii_formatd (str + 4, 15 + 1 - 4,
-		       "%.2g", document->doc->getPDFVersion ());
+		       "%.2g", document->doc->getPDFMajorVersion () + document->doc->getPDFMinorVersion() / 10.0);
       g_value_take_string (value, str);
       break;
+    case PROP_FORMAT_MAJOR:
+      g_value_set_uint (value, document->doc->getPDFMajorVersion ());
+      break;
+    case PROP_FORMAT_MINOR:
+      g_value_set_uint (value, document->doc->getPDFMinorVersion());
+      break;
     case PROP_AUTHOR:
       document->doc->getDocInfo (&obj);
       if (obj.isDict ())
@@ -625,6 +756,8 @@ poppler_document_get_property (GObject    *object,
 	flag |= POPPLER_PERMISSIONS_OK_TO_COPY;
       if (document->doc->okToAddNotes ())
 	flag |= POPPLER_PERMISSIONS_OK_TO_ADD_NOTES;
+      if (document->doc->okToFillForm ())
+        flag |= POPPLER_PERMISSIONS_OK_TO_FILL_FORM;
       g_value_set_flags (value, flag);
       break;
     case PROP_METADATA:
@@ -669,6 +802,24 @@ poppler_document_class_init (PopplerDocumentClass *klass)
 				NULL,
 				G_PARAM_READABLE));
 
+  g_object_class_install_property
+	  (G_OBJECT_CLASS (klass),
+	   PROP_FORMAT_MAJOR,
+	   g_param_spec_uint ("format-major",
+			      "PDF Format Major",
+			      "The PDF major version number of the document",
+			      0, G_MAXUINT, 1,
+			      G_PARAM_READABLE));
+
+  g_object_class_install_property
+	  (G_OBJECT_CLASS (klass),
+	   PROP_FORMAT_MINOR,
+	   g_param_spec_uint ("format-minor",
+			      "PDF Format Minor",
+			      "The PDF minor version number of the document",
+			      0, G_MAXUINT, 0,
+			      G_PARAM_READABLE));
+
   g_object_class_install_property
 	  (G_OBJECT_CLASS (klass),
 	   PROP_AUTHOR,
@@ -805,18 +956,9 @@ struct _PopplerIndexIter
 };
 
 
-GType
-poppler_index_iter_get_type (void)
-{
-  static GType our_type = 0;
-
-  if (our_type == 0)
-    our_type = g_boxed_type_register_static ("PopplerIndexIter",
-					     (GBoxedCopyFunc) poppler_index_iter_copy,
-					     (GBoxedFreeFunc) poppler_index_iter_free);
-
-  return our_type;
-}
+POPPLER_DEFINE_BOXED_TYPE (PopplerIndexIter, poppler_index_iter,
+			   poppler_index_iter_copy,
+			   poppler_index_iter_free)
 
 /**
  * poppler_index_iter_copy:
@@ -1053,18 +1195,9 @@ struct _PopplerFontsIter
 	int index;
 };
 
-GType
-poppler_fonts_iter_get_type (void)
-{
-  static GType our_type = 0;
-
-  if (our_type == 0)
-    our_type = g_boxed_type_register_static ("PopplerFontsIter",
-					     (GBoxedCopyFunc) poppler_fonts_iter_copy,
-					     (GBoxedFreeFunc) poppler_fonts_iter_free);
-
-  return our_type;
-}
+POPPLER_DEFINE_BOXED_TYPE (PopplerFontsIter, poppler_fonts_iter,
+			   poppler_fonts_iter_copy,
+			   poppler_fonts_iter_free)
 
 const char *
 poppler_fonts_iter_get_full_name (PopplerFontsIter *iter)
@@ -1211,7 +1344,7 @@ struct _PopplerFontInfoClass
         GObjectClass parent_class;
 };
 
-G_DEFINE_TYPE (PopplerFontInfo, poppler_font_info, G_TYPE_OBJECT);
+G_DEFINE_TYPE (PopplerFontInfo, poppler_font_info, G_TYPE_OBJECT)
 
 static void poppler_font_info_finalize (GObject *object);
 
@@ -1288,13 +1421,378 @@ poppler_font_info_free (PopplerFontInfo *font_info)
 }
 
 
+/* Optional content (layers) */
+static Layer *
+layer_new (OptionalContentGroup *oc)
+{
+  Layer *layer;
+
+  layer = g_new0 (Layer, 1);
+  layer->oc = oc;
+
+  return layer;
+}
+
+static void
+layer_free (Layer *layer)
+{
+  if (!layer)
+    return;
+
+  if (layer->kids) {
+	  g_list_foreach (layer->kids, (GFunc)layer_free, NULL);
+	  g_list_free (layer->kids);
+  }
+
+  if (layer->label) {
+	  g_free (layer->label);
+  }
+
+  g_free (layer);
+}
+
+static GList *
+get_optional_content_rbgroups (OCGs *ocg)
+{
+  Array *rb;
+  GList *groups = NULL;
+  
+  rb = ocg->getRBGroupsArray ();
+  
+  if (rb) {
+    int i, j;
+
+    for (i = 0; i < rb->getLength (); ++i) {
+      Object obj;
+      Array *rb_array;
+      GList *group = NULL;
+
+      rb->get (i, &obj);
+      if (!obj.isArray ()) {
+        obj.free ();
+	continue;
+      }
+
+      rb_array = obj.getArray ();
+      for (j = 0; j < rb_array->getLength (); ++j) {
+        Object ref;
+	OptionalContentGroup *oc;
+
+	rb_array->getNF (j, &ref);
+	if (!ref.isRef ()) {
+	  ref.free ();
+	  continue;
+	}
+
+	oc = ocg->findOcgByRef (ref.getRef ());
+	group = g_list_prepend (group, oc);
+	ref.free ();
+      }
+      obj.free ();
+
+      groups = g_list_prepend (groups, group);
+    }
+  }
+
+  return groups;
+}
+
+static GList *
+poppler_document_get_layer_rbgroup (PopplerDocument *document,
+				    Layer           *layer)
+{
+  GList *l;
+
+  for (l = document->layers_rbgroups; l && l->data; l = g_list_next (l)) {
+    GList *group = (GList *)l->data;
+
+    if (g_list_find (group, layer->oc))
+      return group;
+  }
+
+  return NULL;
+}
+
+static GList *
+get_optional_content_items_sorted (OCGs *ocg, Layer *parent, Array *order)
+{
+  GList *items = NULL;
+  Layer *last_item = parent;
+  int i;
+
+  for (i = 0; i < order->getLength (); ++i) {
+    Object orderItem;
+      
+    order->get (i, &orderItem);
+
+    if (orderItem.isDict ()) {
+      Object ref;
+      
+      order->getNF (i, &ref);
+      if (ref.isRef ()) {
+        OptionalContentGroup *oc = ocg->findOcgByRef (ref.getRef ());
+	Layer *layer = layer_new (oc);
+
+	items = g_list_prepend (items, layer);
+	last_item = layer;
+      }
+      ref.free ();
+    } else if (orderItem.isArray () && orderItem.arrayGetLength () > 0) {
+      if (!last_item) {
+        last_item = layer_new (NULL);
+	items = g_list_prepend (items, last_item);
+      }
+      last_item->kids = get_optional_content_items_sorted (ocg, last_item, orderItem.getArray ());
+    } else if (orderItem.isString ()) {
+      last_item->label = _poppler_goo_string_to_utf8 (orderItem.getString ());
+    }
+    orderItem.free ();
+  }
+  
+  return g_list_reverse (items);
+}
+		
+static GList *
+get_optional_content_items (OCGs *ocg)
+{
+  Array *order;
+  GList *items = NULL;
+  
+  order = ocg->getOrderArray ();
+
+  if (order) {
+    items = get_optional_content_items_sorted (ocg, NULL, order);
+  } else {
+    GooList *ocgs;
+    int i;
+
+    ocgs = ocg->getOCGs ();
+    
+    for (i = 0; i < ocgs->getLength (); ++i) {
+      OptionalContentGroup *oc = (OptionalContentGroup *) ocgs->get (i);
+      Layer *layer = layer_new (oc);
+      
+      items = g_list_prepend (items, layer);
+    }
+    
+    items = g_list_reverse (items);
+  }
+
+  return items;
+}
+
+static GList *
+poppler_document_get_layers (PopplerDocument *document)
+{
+  if (!document->layers) {
+    Catalog *catalog = document->doc->getCatalog ();
+    OCGs *ocg = catalog->getOptContentConfig ();
+
+    if (!ocg)
+      return NULL;
+    
+    document->layers = get_optional_content_items (ocg);
+    document->layers_rbgroups = get_optional_content_rbgroups (ocg);
+  }
+
+  return document->layers;
+}
+
+static void
+poppler_document_layers_free (PopplerDocument *document)
+{
+  if (!document->layers)
+	  return;
+
+  g_list_foreach (document->layers, (GFunc)layer_free, NULL);
+  g_list_free (document->layers);
+
+  g_list_foreach (document->layers_rbgroups, (GFunc)g_list_free, NULL);
+  g_list_free (document->layers_rbgroups);
+
+  document->layers = NULL;
+  document->layers_rbgroups = NULL;
+}
+
+/* PopplerLayersIter */
+struct _PopplerLayersIter {
+  PopplerDocument *document;
+  GList *items;
+  int index;
+};
+
+POPPLER_DEFINE_BOXED_TYPE (PopplerLayersIter, poppler_layers_iter,
+			   poppler_layers_iter_copy,
+			   poppler_layers_iter_free)
+
+/**
+ * poppler_layers_iter_copy:
+ * @iter: a #PopplerLayersIter
+ * 
+ * Creates a new #PopplerLayersIter as a copy of @iter.  This must be freed with
+ * poppler_layers_iter_free().
+ * 
+ * Return value: a new #PopplerLayersIter
+ **/
+PopplerLayersIter *
+poppler_layers_iter_copy (PopplerLayersIter *iter)
+{
+  PopplerLayersIter *new_iter;
+
+  g_return_val_if_fail (iter != NULL, NULL);
+  
+  new_iter = g_new0 (PopplerLayersIter, 1);
+  *new_iter = *iter;
+  new_iter->document = (PopplerDocument *) g_object_ref (new_iter->document);
+  
+  return new_iter;
+}
+
+/**
+ * poppler_layers_iter_free:
+ * @iter: a #PopplerLayersIter
+ * 
+ * Frees @iter.
+ **/
+void
+poppler_layers_iter_free (PopplerLayersIter *iter)
+{
+  if (iter == NULL)
+    return;
+
+  g_object_unref (iter->document);
+  g_free (iter);
+}
+
+/**
+ * poppler_layers_iter_new:
+ **/
+PopplerLayersIter *
+poppler_layers_iter_new (PopplerDocument *document)
+{
+  PopplerLayersIter *iter;
+  GList *items;
+
+  items = poppler_document_get_layers (document);
+
+  if (!items)
+    return NULL;
+
+  iter = g_new0 (PopplerLayersIter, 1);
+  iter->document = (PopplerDocument *)g_object_ref (document);
+  iter->items = items;
+
+  return iter;
+}
+
+/**
+ * poppler_layers_iter_get_child:
+ * @parent: a #PopplerLayersIter
+ * 
+ * Returns a newly created child of @parent, or %NULL if the iter has no child.
+ * See poppler_layers_iter_new() for more information on this function.
+ * 
+ * Return value: a new #PopplerLayersIter, or %NULL
+ **/
+PopplerLayersIter *
+poppler_layers_iter_get_child (PopplerLayersIter *parent)
+{
+  PopplerLayersIter *child;
+  Layer *layer;
+
+  g_return_val_if_fail (parent != NULL, NULL);
+	
+  layer = (Layer *) g_list_nth_data (parent->items, parent->index);
+  if (!layer || !layer->kids)
+    return NULL;
+
+  child = g_new0 (PopplerLayersIter, 1);
+  child->document = (PopplerDocument *)g_object_ref (parent->document);
+  child->items = layer->kids;
+
+  g_assert (child->items);
+
+  return child;
+}
+
+/**
+ * poppler_layers_iter_get_title:
+ * @iter: a #PopplerLayersIter
+ * 
+ * Returns the title associated with @iter.  It must be freed with
+ * g_free().
+ * 
+ * Return value: a new string containing the @iter's title or %NULL if @iter doesn't have a title.
+ * The returned string should be freed with g_free() when no longer needed. 
+ **/
+gchar *
+poppler_layers_iter_get_title (PopplerLayersIter *iter)
+{
+  Layer *layer;
+  
+  g_return_val_if_fail (iter != NULL, NULL);
+  
+  layer = (Layer *)g_list_nth_data (iter->items, iter->index);
+
+  return layer->label ? g_strdup (layer->label) : NULL;
+}
+
+/**
+ * poppler_layers_iter_get_layer:
+ * @iter: a #PopplerLayersIter
+ * 
+ * Returns the #PopplerLayer associated with @iter.  It must be freed with
+ * poppler_layer_free().
+ * 
+ * Return value: a new #PopplerLayer, or %NULL if there isn't any layer associated with @iter
+ **/
+PopplerLayer *
+poppler_layers_iter_get_layer (PopplerLayersIter *iter)
+{
+  Layer *layer;
+  PopplerLayer *poppler_layer = NULL;
+  
+  g_return_val_if_fail (iter != NULL, NULL);
+  
+  layer = (Layer *)g_list_nth_data (iter->items, iter->index);
+  if (layer->oc) {
+    GList *rb_group = NULL;
+
+    rb_group = poppler_document_get_layer_rbgroup (iter->document, layer);
+    poppler_layer = _poppler_layer_new (iter->document, layer, rb_group);
+  }
+  
+  return poppler_layer;
+}
+
+/**
+ * poppler_layers_iter_next:
+ * @iter: a #PopplerLayersIter
+ * 
+ * Sets @iter to point to the next action at the current level, if valid.  See
+ * poppler_layers_iter_new() for more information.
+ * 
+ * Return value: %TRUE, if @iter was set to the next action
+ **/
+gboolean
+poppler_layers_iter_next (PopplerLayersIter *iter)
+{
+  g_return_val_if_fail (iter != NULL, FALSE);
+
+  iter->index++;
+  if (iter->index >= (gint)g_list_length (iter->items))
+    return FALSE;
+  
+  return TRUE;
+}
+
 typedef struct _PopplerPSFileClass PopplerPSFileClass;
 struct _PopplerPSFileClass
 {
         GObjectClass parent_class;
 };
 
-G_DEFINE_TYPE (PopplerPSFile, poppler_ps_file, G_TYPE_OBJECT);
+G_DEFINE_TYPE (PopplerPSFile, poppler_ps_file, G_TYPE_OBJECT)
 
 static void poppler_ps_file_finalize (GObject *object);
 
@@ -1442,13 +1940,10 @@ poppler_document_get_form_field (PopplerDocument *document,
 
 gboolean
 _poppler_convert_pdf_date_to_gtime (GooString *date,
-				    GTime     *gdate) 
+				    time_t    *gdate) 
 {
-  int year, mon, day, hour, min, sec;
-  int scanned_items;
-  struct tm time;
-  gchar *date_string, *ds;
-  GTime result;
+  gchar *date_string;
+  gboolean retval;
 
   if (date->hasUnicodeMarker()) {
     date_string = g_convert (date->getCString () + 2,
@@ -1457,56 +1952,9 @@ _poppler_convert_pdf_date_to_gtime (GooString *date,
   } else {
     date_string = g_strndup (date->getCString (), date->getLength ());
   }
-  ds = date_string;
-  
-  /* See PDF Reference 1.3, Section 3.8.2 for PDF Date representation */
 
-  if (date_string [0] == 'D' && date_string [1] == ':')
-    date_string += 2;
-	
-  /* FIXME only year is mandatory; parse optional timezone offset */
-  scanned_items = sscanf (date_string, "%4d%2d%2d%2d%2d%2d",
-			  &year, &mon, &day, &hour, &min, &sec);
-  
-  if (scanned_items != 6) {
-    g_free (ds);
-    return FALSE;
-  }
-  
-  /* Workaround for y2k bug in Distiller 3, hoping that it won't
-   * be used after y2.2k */
-  if (year < 1930 && strlen (date_string) > 14) {
-    int century, years_since_1900;
-    scanned_items = sscanf (date_string, "%2d%3d%2d%2d%2d%2d%2d",
-			    &century, &years_since_1900, &mon, &day, &hour, &min, &sec);
-						
-    if (scanned_items != 7) {
-      g_free (ds);
-      return FALSE;
-    }
-    
-    year = century * 100 + years_since_1900;
-  }
+  retval = poppler_date_parse (date_string, gdate);
+  g_free (date_string);
 
-  time.tm_year = year - 1900;
-  time.tm_mon = mon - 1;
-  time.tm_mday = day;
-  time.tm_hour = hour;
-  time.tm_min = min;
-  time.tm_sec = sec;
-  time.tm_wday = -1;
-  time.tm_yday = -1;
-  time.tm_isdst = -1; /* 0 = DST off, 1 = DST on, -1 = don't know */
- 
-  /* compute tm_wday and tm_yday and check date */
-  result = mktime (&time);
-  if (result == (time_t) - 1) {
-    g_free (ds);
-    return FALSE;
-  }
-    
-  g_free (ds);
-  *gdate = result;
-
-  return TRUE;
+  return retval;
 }
