@@ -6,6 +6,20 @@
 //
 //========================================================================
 
+//========================================================================
+//
+// Modified under the Poppler project - http://poppler.freedesktop.org
+//
+// All changes made under the Poppler project to this file are licensed
+// under GPL version 2 or later
+//
+// Copyright (C) 2005 Kristian Høgsberg <krh@redhat.com>
+//
+// To see a description of the changes please see the Changelog file that
+// came with your tarball or type make ChangeLog if you are building from git
+//
+//========================================================================
+
 #ifndef ARRAY_H
 #define ARRAY_H
 
@@ -43,6 +57,7 @@ public:
   // Accessors.
   Object *get(int i, Object *obj);
   Object *getNF(int i, Object *obj);
+  GBool getString(int i, GooString *string);
 
 private:
 
