@@ -2,7 +2,11 @@
 //
 // DCTStream.h
 //
-// Copyright 1996-2003 Glyph & Cog, LLC
+// This file is licensed under the GPLv2 or later
+//
+// Copyright 2005 Jeff Muizelaar <jeff@infidigm.net>
+// Copyright 2005 Martin Kretzschmar <martink@gnome.org>
+// Copyright 2005-2007, 2009 Albert Astals Cid <aacid@kde.org>
 //
 //========================================================================
 
@@ -64,7 +68,8 @@ public:
 private:
   void init();
 
-  unsigned int x;
+  JSAMPLE *current;
+  JSAMPLE *limit;
   struct jpeg_decompress_struct cinfo;
   struct jpeg_error_mgr jerr;
   struct str_src_mgr src;
