@@ -6,6 +6,21 @@
 //
 //========================================================================
 
+//========================================================================
+//
+// Modified under the Poppler project - http://poppler.freedesktop.org
+//
+// Copyright (C) 2005-2007 Kristian Høgsberg <krh@redhat.com>
+// Copyright (C) 2006 Ed Catmur <ed@catmur.co.uk>
+// Copyright (C) 2007-2008 Carlos Garcia Campos <carlosgc@gnome.org>
+// Copyright (C) 2007 Adrian Johnson <ajohnson@redneon.com>
+// Copyright (C) 2008 Albert Astals Cid <aacid@kde.org>
+//
+// To see a description of the changes please see the Changelog file that
+// came with your tarball or type make ChangeLog if you are building from git
+//
+//========================================================================
+
 #ifndef TEXTOUTPUTDEV_H
 #define TEXTOUTPUTDEV_H
 
@@ -26,6 +41,7 @@ class Gfx;
 class GfxFont;
 class GfxState;
 class UnicodeMap;
+class Link;
 
 class TextWord;
 class TextPool;
@@ -41,6 +57,12 @@ class TextSelectionVisitor;
 
 typedef void (*TextOutputFunc)(void *stream, char *text, int len);
 
+enum SelectionStyle {
+  selectionStyleGlyph,
+  selectionStyleWord,
+  selectionStyleLine
+};
+
 //------------------------------------------------------------------------
 // TextFontInfo
 //------------------------------------------------------------------------
@@ -53,11 +75,24 @@ public:
 
   GBool matches(GfxState *state);
 
+#if TEXTOUT_WORD_LIST
+  // Get the font name (which may be NULL).
+  GooString *getFontName() { return fontName; }
+
+  // Get font descriptor flags.
+  GBool isFixedWidth() { return flags & fontFixedWidth; }
+  GBool isSerif() { return flags & fontSerif; }
+  GBool isSymbolic() { return flags & fontSymbolic; }
+  GBool isItalic() { return flags & fontItalic; }
+  GBool isBold() { return flags & fontBold; }
+#endif
+
 private:
 
   GfxFont *gfxFont;
 #if TEXTOUT_WORD_LIST
   GooString *fontName;
+  int flags;
 #endif
 
   friend class TextWord;
@@ -97,7 +132,14 @@ public:
   static int cmpYX(const void *p1, const void *p2);
 
   void visitSelection(TextSelectionVisitor *visitor,
-		      PDFRectangle *selection);
+		      PDFRectangle *selection,
+		      SelectionStyle style);
+
+  // Get the TextFontInfo object associated with this word.
+  TextFontInfo *getFontInfo() { return font; }
+
+  // Get the next TextWord on the linked list.
+  TextWord *getNext() { return next; }
 
 #if TEXTOUT_WORD_LIST
   int getLength() { return len; }
@@ -108,11 +150,16 @@ public:
     { *r = colorR; *g = colorG; *b = colorB; }
   void getBBox(double *xMinA, double *yMinA, double *xMaxA, double *yMaxA)
     { *xMinA = xMin; *yMinA = yMin; *xMaxA = xMax; *yMaxA = yMax; }
+  void getCharBBox(int charIdx, double *xMinA, double *yMinA,
+		   double *xMaxA, double *yMaxA);
   double getFontSize() { return fontSize; }
   int getRotation() { return rot; }
   int getCharPos() { return charPos; }
   int getCharLen() { return charLen; }
+  GBool getSpaceAfter() { return spaceAfter; }
 #endif
+  GBool isUnderlined() { return underlined; }
+  Link *getLink() { return link; }
   double getEdge(int i) { return edge[i]; }
   double getBaseline () { return base; }
   GBool hasSpaceAfter  () { return spaceAfter; }
@@ -145,6 +192,9 @@ private:
          colorB;
 #endif
 
+  GBool underlined;
+  Link *link;
+
   friend class TextPool;
   friend class TextLine;
   friend class TextBlock;
@@ -220,7 +270,17 @@ public:
   void coalesce(UnicodeMap *uMap);
 
   void visitSelection(TextSelectionVisitor *visitor,
-		      PDFRectangle *selection);
+		      PDFRectangle *selection,
+		      SelectionStyle style);
+
+  // Get the head of the linked list of TextWords.
+  TextWord *getWords() { return words; }
+
+  // Get the next TextLine on the linked list.
+  TextLine *getNext() { return next; }
+
+  // Returns true if the last char of the line is a hyphen.
+  GBool isHyphenated() { return hyphenated; }
 
 private:
 
@@ -285,7 +345,19 @@ public:
   GBool isBelow(TextBlock *blk);
 
   void visitSelection(TextSelectionVisitor *visitor,
-		      PDFRectangle *selection);
+		      PDFRectangle *selection,
+		      SelectionStyle style);
+
+  // Get the head of the linked list of TextLines.
+  TextLine *getLines() { return lines; }
+
+  // Get the next TextBlock on the linked list.
+  TextBlock *getNext() { return next; }
+
+  void getBBox(double *xMinA, double *yMinA, double *xMaxA, double *yMaxA)
+    { *xMinA = xMin; *yMinA = yMin; *xMaxA = xMax; *yMaxA = yMax; }
+
+  int getLineCount() { return nLines; }
 
 private:
 
@@ -334,6 +406,12 @@ public:
   // primary axis.
   GBool blockFits(TextBlock *blk, TextBlock *prevBlk);
 
+  // Get the head of the linked list of TextBlocks.
+  TextBlock *getBlocks() { return blocks; }
+
+  // Get the next TextFlow on the linked list.
+  TextFlow *getNext() { return next; }
+
 private:
 
   TextPage *page;		// the parent page
@@ -373,7 +451,7 @@ public:
 
 private:
 
-  GooList *words;
+  GooList *words;			// [TextWord]
 };
 
 #endif // TEXTOUT_WORD_LIST
@@ -388,8 +466,8 @@ public:
   // Constructor.
   TextPage(GBool rawOrderA);
 
-  // Destructor.
-  ~TextPage();
+  void incRefCnt();
+  void decRefCnt();
 
   // Start a new page.
   void startPage(GfxState *state);
@@ -414,8 +492,14 @@ public:
   // Add a word, sorting it into the list of words.
   void addWord(TextWord *word);
 
+  // Add a (potential) underline.
+  void addUnderline(double x0, double y0, double x1, double y1);
+
+  // Add a hyperlink.
+  void addLink(int xMin, int yMin, int xMax, int yMax, Link *link);
+
   // Coalesce strings that look like parts of the same line.
-  void coalesce(GBool physLayout);
+  void coalesce(GBool physLayout, GBool doHTML);
 
   // Find a string.  If <startAtTop> is true, starts looking at the
   // top of the page; else if <startAtLast> is true, starts looking
@@ -436,17 +520,22 @@ public:
 		     double xMax, double yMax);
 
   void visitSelection(TextSelectionVisitor *visitor,
-		      PDFRectangle *selection);
+		      PDFRectangle *selection,
+		      SelectionStyle style);
 
   void drawSelection(OutputDev *out,
 		     double scale,
 		     int rotation,
 		     PDFRectangle *selection,
+		     SelectionStyle style,
 		     GfxColor *glyph_color, GfxColor *box_color);
 
-  GooList *getSelectionRegion(PDFRectangle *selection, double scale);
+  GooList *getSelectionRegion(PDFRectangle *selection,
+			      SelectionStyle style,
+			      double scale);
 
-  GooString *getSelectionText(PDFRectangle *selection);
+  GooString *getSelectionText(PDFRectangle *selection,
+			      SelectionStyle style);
 
   // Find a string by character position and length.  If found, sets
   // the text bounding rectangle and returns true; otherwise returns
@@ -459,6 +548,9 @@ public:
   void dump(void *outputStream, TextOutputFunc outputFunc,
 	    GBool physLayout);
 
+  // Get the head of the linked list of TextFlows.
+  TextFlow *getFlows() { return flows; }
+
 #if TEXTOUT_WORD_LIST
   // Build a flat word list, in content stream order (if
   // this->rawOrder is true), physical layout order (if <physLayout>
@@ -468,7 +560,10 @@ public:
 #endif
 
 private:
-
+  
+  // Destructor.
+  ~TextPage();
+  
   void clear();
   void assignColumns(TextLineFrag *frags, int nFrags, int rot);
   int dumpFragment(Unicode *text, int len, UnicodeMap *uMap, GooString *s);
@@ -504,6 +599,11 @@ private:
          lastFindYMin;
   GBool haveLastFind;
 
+  GooList *underlines;		// [TextUnderline]
+  GooList *links;		// [TextLink]
+
+  int refCnt;
+
   friend class TextLine;
   friend class TextLineFrag;
   friend class TextBlock;
@@ -513,6 +613,33 @@ private:
   friend class TextSelectionDumper;
 };
 
+//------------------------------------------------------------------------
+// ActualText
+//------------------------------------------------------------------------
+
+class ActualText {
+public:
+  // Create an ActualText
+  ActualText(TextPage *out);
+  ~ActualText();
+
+  void addChar(GfxState *state, double x, double y,
+	       double dx, double dy,
+	       CharCode c, int nBytes, Unicode *u, int uLen);
+  void beginMC(Dict *properties);
+  void endMC(GfxState *state);
+
+private:
+  TextPage *text;
+  int actualTextBMCLevel;       // > 0 when inside ActualText span. Incremented
+                                // for each nested BMC inside the span.
+  GooString *actualText;        // replacement text for the span
+  GBool newActualTextSpan;      // true at start of span. used to init the extent
+  double actualText_x, actualText_y; // extent of the text inside the span
+  double actualText_dx, actualText_dy;
+};
+  
+
 //------------------------------------------------------------------------
 // TextOutputDev
 //------------------------------------------------------------------------
@@ -576,6 +703,18 @@ public:
 			double originX, double originY,
 			CharCode c, int nBytes, Unicode *u, int uLen);
 
+  //----- grouping operators
+  virtual void beginMarkedContent(char *name, Dict *properties);
+  virtual void endMarkedContent(GfxState *state);
+
+  //----- path painting
+  virtual void stroke(GfxState *state);
+  virtual void fill(GfxState *state);
+  virtual void eoFill(GfxState *state);
+
+  //----- link borders
+  virtual void processLink(Link *link, Catalog *catalog);
+
   //----- special access
 
   // Find a string.  If <startAtTop> is true, starts looking at the
@@ -605,11 +744,15 @@ public:
 
   void drawSelection(OutputDev *out, double scale, int rotation,
 		     PDFRectangle *selection,
+		     SelectionStyle style,
 		     GfxColor *glyph_color, GfxColor *box_color);
 
-  GooList *getSelectionRegion(PDFRectangle *selection, double scale);
+  GooList *getSelectionRegion(PDFRectangle *selection,
+			      SelectionStyle style,
+			      double scale);
 
-  GooString *getSelectionText(PDFRectangle *selection);
+  GooString *getSelectionText(PDFRectangle *selection,
+			      SelectionStyle style);
 
 #if TEXTOUT_WORD_LIST
   // Build a flat word list, in content stream order (if
@@ -623,6 +766,9 @@ public:
   // transferring ownership to the caller.
   TextPage *takeText();
 
+  // Turn extra processing for HTML conversion on or off.
+  void enableHTMLExtras(GBool doHTMLA) { doHTML = doHTMLA; }
+
 private:
 
   TextOutputFunc outputFunc;	// output function
@@ -633,7 +779,10 @@ private:
   GBool physLayout;		// maintain original physical layout when
 				//   dumping text
   GBool rawOrder;		// keep text in content stream order
+  GBool doHTML;			// extra processing for HTML conversion
   GBool ok;			// set up ok?
+
+  ActualText *actualText;
 };
 
 #endif
