@@ -31,6 +31,7 @@
 #include <Catalog.h>
 #include <ErrorCodes.h>
 #include <TextOutputDev.h>
+#include <Link.h>
 #if defined(HAVE_SPLASH)
 #include <SplashOutputDev.h>
 #include <splash/SplashBitmap.h>
@@ -277,9 +278,9 @@ QValueList<Link*> Page::links() const
         {
           LinkGoToR * g = (LinkGoToR *) a;
           // copy link file
-          const char * fileName = g->getFileName()->getCString();
+          const QString fileName = UnicodeParsedString( g->getFileName() );
           // ceate link: fileName, namedDest, object pointer
-          popplerLink = new LinkGoto( linkArea, (QString)fileName, LinkDestination( LinkDestinationData(g->getDest(), g->getNamedDest(), data->doc->data ) ) );
+          popplerLink = new LinkGoto( linkArea, fileName, LinkDestination( LinkDestinationData(g->getDest(), g->getNamedDest(), data->doc->data ) ) );
         }
         break;
 
@@ -335,17 +336,9 @@ QValueList<Link*> Page::links() const
         break;
 
         case actionMovie:
-/*      TODO this (Movie link)
-          m_type = Movie;
-          LinkMovie * m = (LinkMovie *) a;
-          // copy Movie parameters (2 IDs and a const char *)
-          Ref * r = m->getAnnotRef();
-          m_refNum = r->num;
-          m_refGen = r->gen;
-          copyString( m_uri, m->getTitle()->getCString() );
-*/      break;
         case actionSound:
-        /* TODO: write me */
+        case actionRendition:
+        case actionJavaScript:
         break;
 
         case actionUnknown:
