@@ -10,6 +10,26 @@
 //
 //========================================================================
 
+//========================================================================
+//
+// Modified under the Poppler project - http://poppler.freedesktop.org
+//
+// All changes made under the Poppler project to this file are licensed
+// under GPL version 2 or later
+//
+// Copyright (C) 2005-2009 Albert Astals Cid <aacid@kde.org>
+// Copyright (C) 2008 Kjartan Maraas <kmaraas@gnome.org>
+// Copyright (C) 2008 Boris Toloknov <tlknv@yandex.ru>
+// Copyright (C) 2008 Haruyuki Kawabe <Haruyuki.Kawabe@unisys.co.jp>
+// Copyright (C) 2008 Tomas Are Haavet <tomasare@gmail.com>
+// Copyright (C) 2009 Warren Toomey <wkt@tuhs.org>
+// Copyright (C) 2009 Carlos Garcia Campos <carlosgc@gnome.org>
+//
+// To see a description of the changes please see the Changelog file that
+// came with your tarball or type make ChangeLog if you are building from git
+//
+//========================================================================
+
 #ifdef __GNUC__
 #pragma implementation
 #endif
@@ -34,9 +54,11 @@
 #include "GlobalParams.h"
 #include "HtmlOutputDev.h"
 #include "HtmlFonts.h"
+#include "PNGWriter.h"
 
 int HtmlPage::pgNum=0;
 int HtmlOutputDev::imgNum=1;
+GooList *HtmlOutputDev::imgList=new GooList();
 
 extern double scale;
 extern GBool complexMode;
@@ -81,8 +103,18 @@ HtmlString::HtmlString(GfxState *state, double fontSize, HtmlFontAccu* fonts) {
 
   state->transform(state->getCurX(), state->getCurY(), &x, &y);
   if ((font = state->getFont())) {
-    yMin = y - font->getAscent() * fontSize;
-    yMax = y - font->getDescent() * fontSize;
+    double ascent = font->getAscent();
+    double descent = font->getDescent();
+    if( ascent > 1.05 ){
+        //printf( "ascent=%.15g is too high, descent=%.15g\n", ascent, descent );
+        ascent = 1.05;
+    }
+    if( descent < -0.4 ){
+        //printf( "descent %.15g is too low, ascent=%.15g\n", descent, ascent );
+        descent = -0.4;
+    }
+    yMin = y - ascent * fontSize;
+    yMax = y - descent * fontSize;
     GfxRGB rgb;
     state->getFillRGB(&rgb);
     GooString *name = state->getFont()->getName();
@@ -115,7 +147,7 @@ HtmlString::HtmlString(GfxState *state, double fontSize, HtmlFontAccu* fonts) {
 
 
 HtmlString::~HtmlString() {
-  delete text;
+  gfree(text);
   delete htext;
   gfree(xRight);
 }
@@ -333,6 +365,36 @@ void HtmlPage::endString() {
   curStr = NULL;
 }
 
+static const char *strrstr( const char *s, const char *ss )
+{
+  const char *p = strstr( s, ss );
+  for( const char *pp = p; pp != NULL; pp = strstr( p+1, ss ) ){
+    p = pp;
+  }
+  return p;
+}
+
+static void CloseTags( GooString *htext, GBool &finish_a, GBool &finish_italic, GBool &finish_bold )
+{
+  const char *last_italic = finish_italic && ( finish_bold   || finish_a    ) ? strrstr( htext->getCString(), "<i>" ) : NULL;
+  const char *last_bold   = finish_bold   && ( finish_italic || finish_a    ) ? strrstr( htext->getCString(), "<b>" ) : NULL;
+  const char *last_a      = finish_a      && ( finish_italic || finish_bold ) ? strrstr( htext->getCString(), "<a " ) : NULL;
+  if( finish_a && ( finish_italic || finish_bold ) && last_a > ( last_italic > last_bold ? last_italic : last_bold ) ){
+    htext->append("</a>", 4);
+    finish_a = false;
+  }
+  if( finish_italic && finish_bold && last_italic > last_bold ){
+    htext->append("</i>", 4);
+    finish_italic = false;
+  }
+  if( finish_bold )
+    htext->append("</b>", 4);
+  if( finish_italic )
+    htext->append("</i>", 4);
+  if( finish_a )
+    htext->append("</a>");
+}
+
 void HtmlPage::coalesce() {
   HtmlString *str1, *str2;
   HtmlFont *hfont1, *hfont2;
@@ -462,7 +524,7 @@ void HtmlPage::coalesce() {
 					str1->size * sizeof(double));
       if (addSpace) {
 		  str1->text[str1->len] = 0x20;
-		  str1->htext->append(" ");
+		  str1->htext->append(xml?" ":"&nbsp;");
 		  str1->xRight[str1->len] = str2->xMin;
 		  ++str1->len;
       }
@@ -494,28 +556,24 @@ void HtmlPage::coalesce() {
 	++str1->len;
       }
 
-      /* fix <i> and <b> if str1 and str2 differ */
-      if( hfont1->isBold() && !hfont2->isBold() )
-	str1->htext->append("</b>", 4);
-      if( hfont1->isItalic() && !hfont2->isItalic() )
-	str1->htext->append("</i>", 4);
-      if( !hfont1->isBold() && hfont2->isBold() )
-	str1->htext->append("<b>", 3);
-      if( !hfont1->isItalic() && hfont2->isItalic() )
-	str1->htext->append("<i>", 3);
-
-      /* now handle switch of links */
+      /* fix <i>, <b> if str1 and str2 differ and handle switch of links */
       HtmlLink *hlink1 = str1->getLink();
       HtmlLink *hlink2 = str2->getLink();
-      if( !hlink1 || !hlink2 || !hlink1->isEqualDest(*hlink2) ) {
-	if(hlink1 != NULL )
-	  str1->htext->append("</a>");
-	if(hlink2 != NULL ) {
-	  GooString *ls = hlink2->getLinkStart();
-	  str1->htext->append(ls);
-	  delete ls;
-	}
+      bool switch_links = !hlink1 || !hlink2 || !hlink1->isEqualDest(*hlink2);
+      GBool finish_a = switch_links && hlink1 != NULL;
+      GBool finish_italic = hfont1->isItalic() && ( !hfont2->isItalic() || finish_a );
+      GBool finish_bold   = hfont1->isBold()   && ( !hfont2->isBold()   || finish_a || finish_italic );
+      CloseTags( str1->htext, finish_a, finish_italic, finish_bold );
+      if( switch_links && hlink2 != NULL ) {
+        GooString *ls = hlink2->getLinkStart();
+        str1->htext->append(ls);
+        delete ls;
       }
+      if( ( !hfont1->isItalic() || finish_italic ) && hfont2->isItalic() )
+	    str1->htext->append("<i>", 3);
+      if( ( !hfont1->isBold() || finish_bold ) && hfont2->isBold() )
+	    str1->htext->append("<b>", 3);
+
 
       str1->htext->append(str2->htext);
       // str1 now contains href for link of str2 (if it is defined)
@@ -531,12 +589,10 @@ void HtmlPage::coalesce() {
       delete str2;
     } else { // keep strings separate
 //      printf("no\n"); 
-      if( hfont1->isBold() )
-	str1->htext->append("</b>",4);
-      if( hfont1->isItalic() )
-	str1->htext->append("</i>",4);
-      if(str1->getLink() != NULL )
-	str1->htext->append("</a>");
+      GBool finish_a = str1->getLink() != NULL;
+      GBool finish_bold   = hfont1->isBold();
+      GBool finish_italic = hfont1->isItalic();
+      CloseTags( str1->htext, finish_a, finish_italic, finish_bold );
      
       str1->xMin = curX; str1->yMin = curY; 
       str1 = str2;
@@ -554,12 +610,11 @@ void HtmlPage::coalesce() {
     }
   }
   str1->xMin = curX; str1->yMin = curY;
-  if( hfont1->isBold() )
-    str1->htext->append("</b>",4);
-  if( hfont1->isItalic() )
-    str1->htext->append("</i>",4);
-  if(str1->getLink() != NULL )
-    str1->htext->append("</a>");
+
+  GBool finish_bold   = hfont1->isBold();
+  GBool finish_italic = hfont1->isItalic();
+  GBool finish_a = str1->getLink() != NULL;
+  CloseTags( str1->htext, finish_a, finish_italic, finish_bold );
 
 #if 0 //~ for debugging
   for (str1 = yxStrings; str1; str1 = str1->yxNext) {
@@ -708,11 +763,14 @@ void HtmlPage::dump(FILE *f, int pageNum)
   else
   {
     fprintf(f,"<A name=%d></a>",pageNum);
-    GooString* fName=basename(DocName); 
-    for (int i=1;i<HtmlOutputDev::imgNum;i++)
-      fprintf(f,"<IMG src=\"%s-%d_%d.jpg\"><br>\n",fName->getCString(),pageNum,i);
+    // Loop over the list of image names on this page
+    int listlen=HtmlOutputDev::imgList->getLength();
+    for (int i = 0; i < listlen; i++) {
+      GooString *fName= (GooString *)HtmlOutputDev::imgList->del(0);
+      fprintf(f,"<IMG src=\"%s\"><br>\n",fName->getCString());
+      delete fName;
+    }
     HtmlOutputDev::imgNum=1;
-    delete fName;
 
     GooString* str;
     for(HtmlString *tmp=yxStrings;tmp;tmp=tmp->yxNext){
@@ -1042,8 +1100,9 @@ void HtmlOutputDev::endPage() {
   Links *linksList = catalog->getPage(pageNum)->getLinks(catalog);
   for (int i = 0; i < linksList->getNumLinks(); ++i)
   {
-      processLink(linksList->getLink(i));
+      doProcessLink(linksList->getLink(i));
   }
+  delete linksList;
 
   pages->conv();
   pages->coalesce();
@@ -1083,11 +1142,11 @@ void HtmlOutputDev::drawChar(GfxState *state, double x, double y,
 }
 
 void HtmlOutputDev::drawImageMask(GfxState *state, Object *ref, Stream *str,
-			      int width, int height, GBool invert,
-			      GBool inlineImg) {
+				  int width, int height, GBool invert,
+				  GBool interpolate, GBool inlineImg) {
 
   if (ignore||complexMode) {
-    OutputDev::drawImageMask(state, ref, str, width, height, invert, inlineImg);
+    OutputDev::drawImageMask(state, ref, str, width, height, invert, interpolate, inlineImg);
     return;
   }
   
@@ -1138,9 +1197,13 @@ void HtmlOutputDev::drawImageMask(GfxState *state, Object *ref, Stream *str,
     GooString *imgnum=GooString::fromInt(imgNum);
     // open the image file
     fName->append(pgNum)->append("_")->append(imgnum)->append(".jpg");
+    delete pgNum;
+    delete imgnum;
+
     ++imgNum;
     if (!(f1 = fopen(fName->getCString(), "wb"))) {
       error(-1, "Couldn't open image file '%s'", fName->getCString());
+      delete fName;
       return;
     }
 
@@ -1154,21 +1217,19 @@ void HtmlOutputDev::drawImageMask(GfxState *state, Object *ref, Stream *str,
 
     fclose(f1);
    
-  if (pgNum) delete pgNum;
-  if (imgnum) delete imgnum;
-  if (fName) delete fName;
+  if (fName) imgList->append(fName);
   }
   else {
-    OutputDev::drawImageMask(state, ref, str, width, height, invert, inlineImg);
+    OutputDev::drawImageMask(state, ref, str, width, height, invert, interpolate, inlineImg);
   }
 }
 
 void HtmlOutputDev::drawImage(GfxState *state, Object *ref, Stream *str,
-			  int width, int height, GfxImageColorMap *colorMap,
-			  int *maskColors, GBool inlineImg) {
+			      int width, int height, GfxImageColorMap *colorMap,
+			      GBool interpolate, int *maskColors, GBool inlineImg) {
 
   if (ignore||complexMode) {
-    OutputDev::drawImage(state, ref, str, width, height, colorMap, 
+    OutputDev::drawImage(state, ref, str, width, height, colorMap, interpolate,
 			 maskColors, inlineImg);
     return;
   }
@@ -1224,10 +1285,14 @@ void HtmlOutputDev::drawImage(GfxState *state, Object *ref, Stream *str,
     
     // open the image file
     fName->append(pgNum)->append("_")->append(imgnum)->append(".jpg");
+    delete pgNum;
+    delete imgnum;
+
     ++imgNum;
     
     if (!(f1 = fopen(fName->getCString(), "wb"))) {
       error(-1, "Couldn't open image file '%s'", fName->getCString());
+      delete fName;
       return;
     }
 
@@ -1241,19 +1306,85 @@ void HtmlOutputDev::drawImage(GfxState *state, Object *ref, Stream *str,
     
     fclose(f1);
   
-    delete fName;
-    delete pgNum;
-    delete imgnum;
+    if (fName) imgList->append(fName);
   }
   else {
-    OutputDev::drawImage(state, ref, str, width, height, colorMap,
+#ifdef ENABLE_LIBPNG
+    // Dump the image as a PNG file. Much of the PNG code
+    // comes from an example by Guillaume Cottenceau.
+    Guchar *p;
+    GfxRGB rgb;
+    png_byte *row = (png_byte *) malloc(3 * width);   // 3 bytes/pixel: RGB
+    png_bytep *row_pointer= &row;
+
+    // Create the image filename
+    GooString *fName=new GooString(Docname);
+    fName->append("-");
+    GooString *pgNum= GooString::fromInt(pageNum);
+    GooString *imgnum= GooString::fromInt(imgNum);  
+    fName->append(pgNum)->append("_")->append(imgnum)->append(".png");
+    delete pgNum;
+    delete imgnum;
+
+    // Open the image file
+    if (!(f1 = fopen(fName->getCString(), "wb"))) {
+      error(-1, "Couldn't open image file '%s'", fName->getCString());
+      delete fName;
+      return;
+    }
+
+    PNGWriter *writer = new PNGWriter();
+    if (!writer->init(f1, width, height)) {
+        delete writer;
+        fclose(f1);
+        return;
+    }
+
+    // Initialize the image stream
+    ImageStream *imgStr = new ImageStream(str, width,
+                        colorMap->getNumPixelComps(), colorMap->getBits());
+    imgStr->reset();
+
+    // For each line...
+    for (int y = 0; y < height; y++) {
+
+      // Convert into a PNG row
+      p = imgStr->getLine();
+      for (int x = 0; x < width; x++) {
+        colorMap->getRGB(p, &rgb);
+	// Write the RGB pixels into the row
+	row[3*x]= colToByte(rgb.r);
+	row[3*x+1]= colToByte(rgb.g);
+	row[3*x+2]= colToByte(rgb.b);
+         p += colorMap->getNumPixelComps();
+      }
+
+      if (!writer->writeRow(row_pointer)) {
+        delete writer;
+        fclose(f1);
+        return;
+      }
+    }
+
+    writer->close();
+    delete writer;
+    fclose(f1);
+
+    free(row);
+    imgList->append(fName);
+    ++imgNum;
+    imgStr->close();
+    delete imgStr;
+#else
+    OutputDev::drawImage(state, ref, str, width, height, colorMap, interpolate,
 			 maskColors, inlineImg);
+#endif
   }
 }
 
 
 
-void HtmlOutputDev::processLink(Link* link){
+void HtmlOutputDev::doProcessLink(Link* link){
   double _x1,_y1,_x2,_y2;
   int x1,y1,x2,y2;
   
@@ -1279,10 +1410,11 @@ GooString* HtmlOutputDev::getLinkDest(Link *link,Catalog* catalog){
 	  int page=1;
 	  LinkGoTo *ha=(LinkGoTo *)link->getAction();
 	  LinkDest *dest=NULL;
-	  if (ha->getDest()==NULL) 
-	      dest=catalog->findDest(ha->getNamedDest());
-	  else 
+	  if (ha->getDest()!=NULL)
 	      dest=ha->getDest()->copy();
+	  else if (ha->getNamedDest()!=NULL)
+	      dest=catalog->findDest(ha->getNamedDest());
+	      
 	  if (dest){ 
 	      if (dest->isPageRef()){
 		  Ref pageref=dest->getPageRef();
@@ -1354,7 +1486,7 @@ GooString* HtmlOutputDev::getLinkDest(Link *link,Catalog* catalog){
 		  file->append(GooString::fromInt(page));
 	      }
 	  }
-	  if (printCommands) printf("filename %s\n",file->getCString());
+	  if (printCommands && file) printf("filename %s\n",file->getCString());
 	  return file;
 	  }
       case actionURI:
@@ -1423,7 +1555,7 @@ GBool HtmlOutputDev::dumpDocOutline(Catalog* catalog)
 		}
 		else
 		{
-			GooString *str = basename(Docname);
+			GooString *str = Docname->copy();
 			str->append("-outline.html");
 			output = fopen(str->getCString(), "w");
 			if (output == NULL)
@@ -1475,10 +1607,11 @@ GBool HtmlOutputDev::newOutlineLevel(FILE *output, Object *node, Catalog* catalo
       if (!curr.dictLookup("Dest", &dest)->isNull()) {
 		LinkGoTo *link = new LinkGoTo(&dest);
 		LinkDest *linkdest=NULL;
-		if (link->getDest()==NULL) 
-	  		linkdest=catalog->findDest(link->getNamedDest());
-		else 
-	  		linkdest=link->getDest()->copy();
+		if (link->getDest()!=NULL)
+			linkdest=link->getDest()->copy();
+		else if (link->getNamedDest()!=NULL)
+			linkdest=catalog->findDest(link->getNamedDest());
+			
 		delete link;
 		if (linkdest) { 
 	  		int page;
