@@ -17,6 +17,8 @@
 // Copyright (C) 2006 Rainer Keller <class321@gmx.de>
 // Copyright (C) 2008 Timothy Lee <timothy.lee@siriushk.com>
 // Copyright (C) 2008 Vasile Gaburici <gaburici@cs.umd.edu>
+// Copyright (C) 2009 Carlos Garcia Campos <carlosgc@gnome.org>
+// Copyright (C) 2009 William Bader <williambader@hotmail.com>
 //
 // To see a description of the changes please see the Changelog file that
 // came with your tarball or type make ChangeLog if you are building from git
@@ -59,7 +61,7 @@ ImageOutputDev::~ImageOutputDev() {
 
 void ImageOutputDev::drawImageMask(GfxState *state, Object *ref, Stream *str,
 				   int width, int height, GBool invert,
-				   GBool inlineImg) {
+				   GBool interpolate, GBool inlineImg) {
   FILE *f;
   int c;
   int size, i;
@@ -116,7 +118,7 @@ void ImageOutputDev::drawImageMask(GfxState *state, Object *ref, Stream *str,
 void ImageOutputDev::drawImage(GfxState *state, Object *ref, Stream *str,
 			       int width, int height,
 			       GfxImageColorMap *colorMap,
-			       int *maskColors, GBool inlineImg) {
+			       GBool interpolate, int *maskColors, GBool inlineImg) {
   FILE *f;
   ImageStream *imgStr;
   Guchar *p;
@@ -217,6 +219,7 @@ void ImageOutputDev::drawImage(GfxState *state, Object *ref, Stream *str,
 	p += colorMap->getNumPixelComps();
       }
     }
+    imgStr->close();
     delete imgStr;
 
     fclose(f);
@@ -225,19 +228,19 @@ void ImageOutputDev::drawImage(GfxState *state, Object *ref, Stream *str,
 
 void ImageOutputDev::drawMaskedImage(
   GfxState *state, Object *ref, Stream *str,
-  int width, int height, GfxImageColorMap *colorMap,
-  Stream *maskStr, int maskWidth, int maskHeight, GBool maskInvert) {
-  drawImage(state, ref, str, width, height, colorMap, NULL, gFalse);
-  drawImageMask(state, ref, maskStr, maskWidth, maskHeight,
-		maskInvert, gFalse);
+  int width, int height, GfxImageColorMap *colorMap, GBool interpolate,
+  Stream *maskStr, int maskWidth, int maskHeight, GBool maskInvert, GBool maskInterpolate) {
+  drawImage(state, ref, str, width, height, colorMap, interpolate, NULL, gFalse);
+  drawImageMask(state, ref, maskStr, maskWidth, maskHeight, maskInvert,
+		maskInterpolate, gFalse);
 }
 
 void ImageOutputDev::drawSoftMaskedImage(
   GfxState *state, Object *ref, Stream *str,
-  int width, int height, GfxImageColorMap *colorMap,
+  int width, int height, GfxImageColorMap *colorMap, GBool interpolate,
   Stream *maskStr, int maskWidth, int maskHeight,
-  GfxImageColorMap *maskColorMap) {
-  drawImage(state, ref, str, width, height, colorMap, NULL, gFalse);
+  GfxImageColorMap *maskColorMap, GBool maskInterpolate) {
+  drawImage(state, ref, str, width, height, colorMap, interpolate, NULL, gFalse);
   drawImage(state, ref, maskStr, maskWidth, maskHeight,
-	    maskColorMap, NULL, gFalse);
+	    maskColorMap, maskInterpolate, NULL, gFalse);
 }
